@@ -18,11 +18,12 @@
 Unit tests for session management and API invocation classes.
 """
 
+from datetime import datetime
 from eventlet import greenthread
 import mock
-import six
 import suds
 
+from oslo_context import context
 from oslo_vmware import api
 from oslo_vmware import exceptions
 from oslo_vmware import pbm
@@ -101,7 +102,8 @@ class VMwareAPISessionTest(base.TestCase):
     SERVER_IP = '10.1.2.3'
     PORT = 443
     USERNAME = 'admin'
-    PASSWORD = 'password'
+    PASSWORD = 'password'  # nosec
+    POOL_SIZE = 15
 
     def setUp(self):
         super(VMwareAPISessionTest, self).setUp()
@@ -122,17 +124,22 @@ class VMwareAPISessionTest(base.TestCase):
                                     _create_session,
                                     port=VMwareAPISessionTest.PORT,
                                     cacert=self.cert_mock,
-                                    insecure=False)
+                                    insecure=False,
+                                    pool_size=VMwareAPISessionTest.POOL_SIZE)
 
     def test_vim(self):
         api_session = self._create_api_session(False)
         api_session.vim
-        self.VimMock.assert_called_with(protocol=api_session._scheme,
-                                        host=VMwareAPISessionTest.SERVER_IP,
-                                        port=VMwareAPISessionTest.PORT,
-                                        wsdl_url=api_session._vim_wsdl_loc,
-                                        cacert=self.cert_mock,
-                                        insecure=False)
+        self.VimMock.assert_called_with(
+            protocol=api_session._scheme,
+            host=VMwareAPISessionTest.SERVER_IP,
+            port=VMwareAPISessionTest.PORT,
+            wsdl_url=api_session._vim_wsdl_loc,
+            cacert=self.cert_mock,
+            insecure=False,
+            pool_maxsize=VMwareAPISessionTest.POOL_SIZE,
+            connection_timeout=None,
+            op_id_prefix='oslo.vmware')
 
     @mock.patch.object(pbm, 'Pbm')
     def test_pbm(self, pbm_mock):
@@ -165,7 +172,7 @@ class VMwareAPISessionTest(base.TestCase):
         session_manager = vim_obj.service_content.sessionManager
         vim_obj.Login.assert_called_once_with(
             session_manager, userName=VMwareAPISessionTest.USERNAME,
-            password=VMwareAPISessionTest.PASSWORD)
+            password=VMwareAPISessionTest.PASSWORD, locale='en')
         self.assertFalse(vim_obj.TerminateSession.called)
         self.assertEqual(session.key, api_session._session_id)
         pbm.set_soap_cookie.assert_called_once_with(cookie)
@@ -189,7 +196,7 @@ class VMwareAPISessionTest(base.TestCase):
             userName=VMwareAPISessionTest.USERNAME)
         vim_obj.Login.assert_called_once_with(
             session_manager, userName=VMwareAPISessionTest.USERNAME,
-            password=VMwareAPISessionTest.PASSWORD)
+            password=VMwareAPISessionTest.PASSWORD, locale='en')
         self.assertEqual(new_session_key, api_session._session_id)
 
     def test_create_session_with_existing_active_session(self):
@@ -249,7 +256,7 @@ class VMwareAPISessionTest(base.TestCase):
         session_manager = vim_obj.service_content.sessionManager
         vim_obj.Login.assert_called_once_with(
             session_manager, userName=VMwareAPISessionTest.USERNAME,
-            password=VMwareAPISessionTest.PASSWORD)
+            password=VMwareAPISessionTest.PASSWORD, locale='en')
         api_session.logout()
         vim_obj.Logout.assert_called_once_with(
             session_manager)
@@ -326,7 +333,7 @@ class VMwareAPISessionTest(base.TestCase):
         expected_str = "%s\nFaults: %s\nDetails: %s" % (fault_string,
                                                         fault_list,
                                                         details_str)
-        self.assertEqual(expected_str, six.text_type(e))
+        self.assertEqual(expected_str, str(e))
         self.assertEqual(details, e.details)
 
     def test_invoke_api_with_empty_response(self):
@@ -373,7 +380,23 @@ class VMwareAPISessionTest(base.TestCase):
             userName=api_session._session_username)
         api_session._create_session.assert_called_once_with()
 
-    def test_wait_for_task(self):
+    def test_invoke_api_with_unknown_fault(self):
+        api_session = self._create_api_session(True)
+        fault_list = ['NotAFile']
+
+        module = mock.Mock()
+        module.api.side_effect = exceptions.VimFaultException(fault_list,
+                                                              'Not a file.')
+        ex = self.assertRaises(exceptions.VimFaultException,
+                               api_session.invoke_api,
+                               module,
+                               'api')
+        self.assertEqual(fault_list, ex.fault_list)
+
+    @mock.patch.object(context, 'get_current')
+    def test_wait_for_task(self, mock_curr_ctx):
+        ctx = mock.Mock()
+        mock_curr_ctx.return_value = ctx
         api_session = self._create_api_session(True)
         task_info_list = [('queued', 0), ('running', 40), ('success', 100)]
         task_info_list_size = len(task_info_list)
@@ -382,6 +405,8 @@ class VMwareAPISessionTest(base.TestCase):
             (state, progress) = task_info_list.pop(0)
             task_info = mock.Mock()
             task_info.progress = progress
+            task_info.queueTime = datetime(2016, 12, 6, 15, 29, 43, 79060)
+            task_info.completeTime = datetime(2016, 12, 6, 15, 29, 50, 79060)
             task_info.state = state
             return task_info
 
@@ -394,11 +419,45 @@ class VMwareAPISessionTest(base.TestCase):
         api_session.invoke_api.assert_called_with(vim_util,
                                                   'get_object_property',
                                                   api_session.vim, task,
-                                                  'info')
+                                                  'info',
+                                                  skip_op_id=True)
         self.assertEqual(task_info_list_size,
                          api_session.invoke_api.call_count)
+        mock_curr_ctx.assert_called_once()
+        self.assertEqual(3, ctx.update_store.call_count)
 
-    def test_wait_for_task_with_error_state(self):
+    @mock.patch.object(context, 'get_current', return_value=None)
+    def test_wait_for_task_no_ctx(self, mock_curr_ctx):
+        api_session = self._create_api_session(True)
+        task_info_list = [('queued', 0), ('running', 40), ('success', 100)]
+        task_info_list_size = len(task_info_list)
+
+        def invoke_api_side_effect(module, method, *args, **kwargs):
+            (state, progress) = task_info_list.pop(0)
+            task_info = mock.Mock()
+            task_info.progress = progress
+            task_info.queueTime = datetime(2016, 12, 6, 15, 29, 43, 79060)
+            task_info.completeTime = datetime(2016, 12, 6, 15, 29, 50, 79060)
+            task_info.state = state
+            return task_info
+
+        api_session.invoke_api = mock.Mock(side_effect=invoke_api_side_effect)
+        task = mock.Mock()
+        with mock.patch.object(greenthread, 'sleep'):
+            ret = api_session.wait_for_task(task)
+            self.assertEqual('success', ret.state)
+            self.assertEqual(100, ret.progress)
+        api_session.invoke_api.assert_called_with(vim_util,
+                                                  'get_object_property',
+                                                  api_session.vim, task,
+                                                  'info',
+                                                  skip_op_id=True)
+        self.assertEqual(task_info_list_size,
+                         api_session.invoke_api.call_count)
+        mock_curr_ctx.assert_called_once()
+
+    @mock.patch.object(context, 'get_current')
+    def test_wait_for_task_with_error_state(self, mock_curr_ctx):
         api_session = self._create_api_session(True)
         task_info_list = [('queued', 0), ('running', 40), ('error', -1)]
         task_info_list_size = len(task_info_list)
@@ -413,17 +472,20 @@ class VMwareAPISessionTest(base.TestCase):
         api_session.invoke_api = mock.Mock(side_effect=invoke_api_side_effect)
         task = mock.Mock()
         with mock.patch.object(greenthread, 'sleep'):
-            self.assertRaises(exceptions.VMwareDriverException,
+            self.assertRaises(exceptions.VimFaultException,
                               api_session.wait_for_task,
                               task)
         api_session.invoke_api.assert_called_with(vim_util,
                                                   'get_object_property',
                                                   api_session.vim, task,
-                                                  'info')
+                                                  'info',
+                                                  skip_op_id=True)
         self.assertEqual(task_info_list_size,
                          api_session.invoke_api.call_count)
+        mock_curr_ctx.assert_called_once()
 
-    def test_wait_for_task_with_invoke_api_exception(self):
+    @mock.patch.object(context, 'get_current')
+    def test_wait_for_task_with_invoke_api_exception(self, mock_curr_ctx):
         api_session = self._create_api_session(True)
         api_session.invoke_api = mock.Mock(
             side_effect=exceptions.VimException(None))
@@ -435,7 +497,9 @@ class VMwareAPISessionTest(base.TestCase):
         api_session.invoke_api.assert_called_once_with(vim_util,
                                                        'get_object_property',
                                                        api_session.vim, task,
-                                                       'info')
+                                                       'info',
+                                                       skip_op_id=True)
+        mock_curr_ctx.assert_called_once()
 
     def test_wait_for_lease_ready(self):
         api_session = self._create_api_session(True)
@@ -452,7 +516,8 @@ class VMwareAPISessionTest(base.TestCase):
         api_session.invoke_api.assert_called_with(vim_util,
                                                   'get_object_property',
                                                   api_session.vim, lease,
-                                                  'state')
+                                                  'state',
+                                                  skip_op_id=True)
         self.assertEqual(num_states, api_session.invoke_api.call_count)
 
     def test_wait_for_lease_ready_with_error_state(self):
@@ -469,7 +534,8 @@ class VMwareAPISessionTest(base.TestCase):
                               api_session.wait_for_lease_ready,
                               lease)
         exp_calls = [mock.call(vim_util, 'get_object_property',
-                               api_session.vim, lease, 'state')] * 2
+                               api_session.vim, lease, 'state',
+                               skip_op_id=True)] * 2
         exp_calls.append(mock.call(vim_util, 'get_object_property',
                                    api_session.vim, lease, 'error'))
         self.assertEqual(exp_calls, api_session.invoke_api.call_args_list)
@@ -488,7 +554,8 @@ class VMwareAPISessionTest(base.TestCase):
         api_session.invoke_api.assert_called_once_with(vim_util,
                                                        'get_object_property',
                                                        api_session.vim,
-                                                       lease, 'state')
+                                                       lease, 'state',
+                                                       skip_op_id=True)
 
     def test_wait_for_lease_ready_with_invoke_api_exception(self):
         api_session = self._create_api_session(True)
@@ -500,7 +567,7 @@ class VMwareAPISessionTest(base.TestCase):
                           lease)
         api_session.invoke_api.assert_called_once_with(
             vim_util, 'get_object_property', api_session.vim, lease,
-            'state')
+            'state', skip_op_id=True)
 
     def _poll_task_well_known_exceptions(self, fault,
                                          expected_exception):
@@ -521,21 +588,25 @@ class VMwareAPISessionTest(base.TestCase):
         with (
             mock.patch.object(api_session, 'invoke_api', fake_invoke_api)
         ):
+            fake_task = mock.Mock()
+            fake_task.value = 'task-1'
+            ctx = mock.Mock()
             self.assertRaises(expected_exception,
                               api_session._poll_task,
-                              'fake-task')
+                              fake_task,
+                              ctx)
 
     def test_poll_task_well_known_exceptions(self):
-        for k, v in six.iteritems(exceptions._fault_classes_registry):
+        for k, v in exceptions._fault_classes_registry.items():
             self._poll_task_well_known_exceptions(k, v)
 
     def test_poll_task_unknown_exception(self):
         _unknown_exceptions = {
-            'NoDiskSpace': exceptions.VMwareDriverException,
-            'RuntimeFault': exceptions.VMwareDriverException
+            'NotAFile': exceptions.VimFaultException,
+            'RuntimeFault': exceptions.VimFaultException
         }
 
-        for k, v in six.iteritems(_unknown_exceptions):
+        for k, v in _unknown_exceptions.items():
             self._poll_task_well_known_exceptions(k, v)
 
     def test_update_pbm_wsdl_loc(self):
